'
' Demonstrate input operation on a Microchip MCP23S17 16-bit SPI I/O expander
'
    Include "SPI.inc"         ' Load the SPI macros into the program

    Symbol MCP_CS = PORTC.2   ' MCP23S17 CS line connects here

    Dim bTemp As Byte         ' Used to hold the input value
'
' Main program starts here
'
    High MCP_CS               ' Disable the SPI interface for now
'
' Open the SPI peripheral:
'                       Fosc/16 (4MHz operation)
'                       SPI mode 0:0
'                       Sample at end of the clock cycle
    OpenSPI(SPI_FOSC_16, MODE_00, SMPEND)
'
' Initialise the MCP23S17
'
' At initialisation, Chip_Address must be forced to "000"
' regardless of actual Mcpchip_Address value.
' After initialisation, once IOCON.HAEN is enabled,
' Mcpchip_Address is taken into account by the chip.
'
    Clear MCP_CS          ' Enable the SPI interface
    DelayCS 1             ' A one cycle delay before starting
    WriteSPI(%11110001)   ' Force ChipAdd to 000
    WriteSPI($0A)         ' Writing to IOCONA writes also to IOCONB
    WriteSPI(%00011000)   ' Init value for IOCON
    Set MCP_CS            ' Disable the SPI interface
'
' Write to the Tris of PortB and set it to all outputs
'
    Clear MCP_CS          ' Enable the SPI interface
    DelayCS 1             ' A one cycle delay before starting
    WriteSPI(%01000000)   ' Target address 000 and set the R/W Line To Write (0)
    WriteSPI($01)         ' We want to access the IODIRB register
    WriteSPI(%00000000)   ' Set to all outputs
    Set MCP_CS            ' Disable the SPI interface
'
' Write to the Tris of PortA and set it to all inputs
'
    Clear MCP_CS          ' Enable the SPI interface
    DelayCS 1             ' A one cycle delay before starting
    WriteSPI(%01000000)   ' Target address 000 and set the R/W Line To Write (0)
    WriteSPI($00)         ' We want to access the IODIRB register
    WriteSPI(%11111111)   ' Set to all inputs
    Set MCP_CS            ' Disable the SPI interface
'
' Enable pull-up resistors on PortA
'
    Clear MCP_CS          ' Enable the SPI interface
    DelayCS 1             ' A one cycle delay before starting
    WriteSPI(%01000000)   ' Target address 000 and set the R/W Line To Write (0)
    WriteSPI($0C)         ' We want to access the GPPUA register
    WriteSPI(%11111111)   ' Set all outputs high
    Set MCP_CS            ' Disable the SPI interface
'
' Display the contents of PortA inputs on PortB outputs
'
    While 1 = 1               ' Create an infinite loop
        '
        ' Read the condition of PortA
        '
        Clear MCP_CS          ' Enable the SPI interface
        DelayCS 1             ' A one cycle delay before starting
        WriteSPI(%01000001)   ' Target address 000 and set the R/W line to Read (1)
        WriteSPI($12)         ' We want to access the GPIOA register
        bTemp = ReadSPI()     ' Read from the SPI interface
        Set MCP_CS            ' Disable the SPI interface
        '
        ' Write the contents of bTemp to the Latch of PortB
        '
        Clear MCP_CS          ' Enable the SPI interface
        DelayCS 1             ' A one cycle delay before starting
        WriteSPI(%01000000)   ' Target address 000 and set the R/W Line To Write (0)
        WriteSPI($15)         ' We want to access the OLATB register
        WriteSPI(~bTemp)      ' Write complement of bTemp to the latches of PortB
        Set MCP_CS            ' Disable the SPI interface
        DelayMS 100
    Wend
